// MIT License
//
// Copyright (c) 2024 vinchent <vinchent@vinchent.xyz>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

package usecase

import (
	"context"
	"fmt"

	"git.vinchent.xyz/vinchent/howmuch/internal/howmuch/model"
	"git.vinchent.xyz/vinchent/howmuch/internal/howmuch/usecase/repo"
	"git.vinchent.xyz/vinchent/howmuch/internal/pkg/log"
)

type userUsecase struct {
	userRepo repo.UserRepository
	dbRepo   repo.DBRepository
}

type User interface {
	Create(ctx context.Context, u *model.User) (*model.User, error)
}

func NewUserUsecase(r repo.UserRepository, d repo.DBRepository) User {
	return &userUsecase{
		userRepo: r,
		dbRepo:   d,
	}
}

func (uuc *userUsecase) Create(ctx context.Context, u *model.User) (*model.User, error) {
	data, err := uuc.dbRepo.Transaction(
		ctx,
		func(txCtx context.Context, tx interface{}) (interface{}, error) {
			// TODO: should check if the user exists
			// DB will return an error since we have set email to UNIQUE.
			// But we may not want to expose the exact db error.

			u, err := uuc.userRepo.Create(txCtx, tx, u)
			if err != nil {
				return nil, err
			}

			// TODO: App log, maybe can be sent to some third party service.
			log.InfoLog(
				"created new user",
				"email",
				u.Email,
				"name",
				fmt.Sprintf("%s %s", u.FirstName, u.LastName),
			)

			return u, err
		},
	)
	if err != nil {
		// TODO: We should wrap the error at service level
		return nil, err
	}

	user := data.(*model.User)

	return user, nil
}
